#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys, os, getopt
import serial
from .serial_terminal import SerialTerminal
from ._version import __version__ as version

help_string = 'Usage: %s [option] <parameter>\nThe option and option\'s parameter list below:\n' \
    'Ex:\n' \
    '\t%s -p COM3 -b 115200\n\n' \
    '-c\n' \
    '\tAdd CR before LF when you change a new line.\n' \
    \
    '-p | --port\n' \
    '\t[ default value: None ]\n' \
    '\tSerial port name.\n' \
    \
    '-b | --baud-rate\n' \
    '\t[ default value: 115200 ]\n' \
    '\tBaud rate.\n' \
    \
    '-w | --data-bits\n' \
    '\t[ default value: 8 ]\n' \
    '\t< 8 | 7 >\n' \
    '\tData bits.\n' \
    \
    '-r | --parity\n' \
    '\t[ default value: None ]\n' \
    '\t< None | Even | Odd | Space | Mark >\n' \
    '\tParity.\n' \
    \
    '-s | --stop-bits\n' \
    '\t[ default value: 1 ]\n' \
    '\t< 1 | 2 | 1.5 >\n' \
    '\tStop bits.\n' \
    \
    '-x | --soft-flow-ctl\n' \
    '\t[ default value: off ]\n' \
    '\t< on | off >\n' \
    '\tSoftware flow control switch.\n' \
    \
    '-f | --hard-flow-ctl\n' \
    '\t[ default value: off ]\n' \
    '\t< on | off >\n' \
    '\tHardware flow control switch.\n' \
    \
    '\nversion %s'

def usage():
    print(help_string % (sys.argv[0], sys.argv[0], version))

valid_parity = {
    'NONE'  : serial.PARITY_NONE,
    'EVEN'  : serial.PARITY_EVEN,
    'ODD'   : serial.PARITY_ODD,
    'MARK'  : serial.PARITY_MARK,
    'SPACE' : serial.PARITY_SPACE
}

valid_stop_bits = {
    '1'     : serial.STOPBITS_ONE,
    '1.5'   : serial.STOPBITS_ONE_POINT_FIVE,
    '2'     : serial.STOPBITS_TWO
}

def open_serial_port(config):
    if config['data-bits'] == 7:
        bytesize = serial.SEVENBITS
    else:
        bytesize = serial.EIGHTBITS
    parity = valid_parity[config['parity']]
    stopbits = valid_stop_bits[config['stop-bits']]
    if config['soft-flow-ctl']:
        xonxoff = True
    else:
        xonxoff = False
    if config['hard-flow-ctl']:
        rtscts = True
    else:
        rtscts = False
    return serial.Serial(port=config['port'], baudrate=config['baud-rate'],\
        bytesize=bytesize, parity=parity, stopbits=stopbits,\
        xonxoff=xonxoff, rtscts=rtscts, timeout=0)

def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    config = {}
    try:
        opts, _ = getopt.getopt(argv, "hcp:b:w:r:s:x:f:", \
            ["port=","baud-rate=","data-bits=","parity=","stop-bits=","soft-flow-ctl=","hard-flow-ctl="])
    except getopt.GetoptError:
        usage()
        sys.exit(1)

    config['port'] = None
    config['baud-rate'] = 115200
    config['data-bits'] = 8
    config['parity'] = 'NONE'
    config['stop-bits'] = '1'
    config['soft-flow-ctl'] = False
    config['hard-flow-ctl'] = False
    addcr = False

    for opt, arg in opts:
        if opt == '-h':
            usage()
            sys.exit()
        elif opt == '-c':
            addcr = True
        elif opt in ("-p", "--port"):
            config['port'] = arg
        elif opt in ("-b", "--baud-rate"):
            config['baud-rate'] = int(arg)
        elif opt in ("-w", "--data-bits"):
            if int(arg) == 7:
                bits = 7
            else:
                bits = 8
            config['data-bits'] = bits
        elif opt in ("-r", "--parity"):
            parity = 'NONE'
            for item in valid_parity.keys():
                if item == arg.upper():
                    parity = arg.upper()
            config['parity'] = parity
        elif opt in ("-s", "--stop-bits"):
            stop_bits = '1'
            for item in valid_stop_bits.keys():
                if item == arg:
                    stop_bits = arg
            config['stop-bits'] = stop_bits
        elif opt in ("-x", "--soft-flow-ctl"):
            if arg.lower() == 'on':
                config['soft-flow-ctl'] = True
            else:
                config['soft-flow-ctl'] = False
        elif opt in ("-f", "--hard-flow-ctl"):
            if arg.lower() == 'on':
                config['hard-flow-ctl'] = True
            else:
                config['hard-flow-ctl'] = False

    for key in config:
        print('key:', key)
        print('value: %s' % str(config[key]))
    port = open_serial_port(config)
    if not port.is_open:
        print('serial port cannot open, please check your port settings!')
        sys.exit(1)
    terminal = SerialTerminal(port)
    terminal.start()

    while terminal.is_alive():
        char = sys.stdin.read(1)
        if char == '\x03': # Ctl-C
            break
        if addcr and char == '\n':
            terminal.write('\r')
        terminal.write(char)

    terminal.abort()
    port.close()

if __name__ == "__main__":
    main(sys.argv[1:])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys, os
import serial
import threading
import time
import signal
import builtins


def print(*args):
    builtins.print(*args, sep=' ', end='', file=None, flush=True)

exit_terminal = False

def exit_sig_handler(sig, frame):
    print('Got signal to exit')
    global exit_terminal
    exit_terminal = True

def serial_handler(option):
    while not exit_terminal:
        operation = False
        line = option['port'].readline().decode('utf-8')
        if len(line) > 0:
            # pexpect will change the LF to CRLF.
            # If the original string is CRLF, it will become CRCRLF in pexpect.
            if os.name == 'nt':
                if len(line) > 2 and line[-1] == '\n' and line[-2] == '\r':
                    line = line[:-2] + '\n'
            print(line)
            operation = True
        if operation == False:
            time.sleep(0.1)

class SerialTerminal():
    """This module provides a simple terminal which can communicate with serial device.
    """
    option = {
        'port'              : None
    }

    def __init__(self, port):
        """[ parameters ]
        port : pyserial's instance
        """
        if not port.is_open:
            raise ValueError('serial port doesn\'t open')
        self.option['port'] = port
        global exit_terminal
        exit_terminal = False
        signal.signal(signal.SIGINT, exit_sig_handler)

    def start(self):
        self.serial_thread = threading.Thread(target = serial_handler, args = (self.option,))
        self.serial_thread.start()

    def write(self, char):
        self.option['port'].write(char.encode('utf-8'))

    def abort(self):
        global exit_terminal
        exit_terminal = True
        self.serial_thread.join()

    def is_alive(self):
        return not exit_terminal

if __name__ == '__main__':
    port = serial.Serial('COM3', 115200, timeout=0)
    terminal = SerialTerminal(port)
    terminal.start()

    while terminal.is_alive():
        char = sys.stdin.read(1)
        if char == '\x03': # Ctl-C
            break
        if char == '\n':
            terminal.write('\r')
        terminal.write(char)

    terminal.abort()
    port.close()



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

def get_version():
    version_file = "pexpect_serial_terminal/_version.py"
    with open(version_file) as f:
        exec(compile(f.read(), version_file, "exec"))
    return locals()["__version__"]

setup(
    name='pexpect-serial-terminal',
    version=get_version(),
    author='Johnny Chiang',
    author_email='johnny641119@gmail.com',
    description='serial terminal for pexpect on both windows and linux platforms',
    license='MIT License',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/angryisland/pexpect-serial-terminal',
    packages=find_packages(exclude=('tests', 'docs', 'examples')),
    python_requires=">=3.0",
    install_requires=['pyserial'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)